<?php

namespace Moblyze\FinancialPlan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\FinancialPlan\Http\Requests\FinancialPlanRequest;
use Moblyze\FinancialPlan\Interfaces\FinancialPlanRepositoryInterface;
use Moblyze\FinancialPlan\Models\FinancialPlan;

/**
 * Resource controller class for financial_plan.
 */
class FinancialPlanResourceController extends BaseController
{

    /**
     * Initialize financial_plan resource controller.
     *
     * @param type FinancialPlanRepositoryInterface $financial_plan
     *
     * @return null
     */
    public function __construct(FinancialPlanRepositoryInterface $financial_plan)
    {
        parent::__construct();
        $this->repository = $financial_plan;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\FinancialPlan\Repositories\Criteria\FinancialPlanResourceCriteria::class);
    }

    /**
     * Display a list of financial_plan.
     *
     * @return Response
     */
    public function index(FinancialPlanRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\FinancialPlan\Repositories\Presenter\FinancialPlanPresenter::class)
                ->$function();
        }

        $financial_plans = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('financial_plan::financial_plan.names'))
            ->view('financial_plan::financial_plan.index', true)
            ->data(compact('financial_plans', 'view'))
            ->output();
    }

    /**
     * Display financial_plan.
     *
     * @param Request $request
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function show(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {

        if ($financial_plan->exists) {
            $view = 'financial_plan::financial_plan.show';
        } else {
            $view = 'financial_plan::financial_plan.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('financial_plan::financial_plan.name'))
            ->data(compact('financial_plan'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new financial_plan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FinancialPlanRequest $request)
    {

        $financial_plan = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('financial_plan::financial_plan.name')) 
            ->view('financial_plan::financial_plan.create', true) 
            ->data(compact('financial_plan'))
            ->output();
    }

    /**
     * Create new financial_plan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FinancialPlanRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $financial_plan                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('financial_plan::financial_plan.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/financial_plan/financial_plan'))
                ->redirect();
        }

    }

    /**
     * Show financial_plan for editing.
     *
     * @param Request $request
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function edit(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('financial_plan::financial_plan.name'))
            ->view('financial_plan::financial_plan.edit', true)
            ->data(compact('financial_plan'))
            ->output();
    }

    /**
     * Update the financial_plan.
     *
     * @param Request $request
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function update(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        try {
            $attributes = $request->all();

            $financial_plan->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('financial_plan::financial_plan.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the financial_plan.
     *
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function destroy(FinancialPlanRequest $request, FinancialPlan $financial_plan)
    {
        try {

            $financial_plan->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('financial_plan::financial_plan.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('financial_plan/financial_plan/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('financial_plan/financial_plan/' . $financial_plan->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple financial_plan.
     *
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function delete(FinancialPlanRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('financial_plan::financial_plan.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('financial_plan/financial_plan'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/financial_plan/financial_plan'))
                ->redirect();
        }

    }

    /**
     * Restore deleted financial_plans.
     *
     * @param Model   $financial_plan
     *
     * @return Response
     */
    public function restore(FinancialPlanRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('financial_plan::financial_plan.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/financial_plan/financial_plan'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/financial_plan/financial_plan/'))
                ->redirect();
        }

    }

}
